import 'package:flutter/material.dart';
import 'package:xinlake_text/validator.dart';

class ValidatorDemo extends StatefulWidget {
  const ValidatorDemo({Key? key}) : super(key: key);

  @override
  State<ValidatorDemo> createState() => _ValidatorState();
}

class _ValidatorState extends State<ValidatorDemo> {
  @override
  Widget build(BuildContext context) {
    const InputDecoration _decoration = InputDecoration(
      contentPadding: EdgeInsets.only(top: 8, bottom: 4),
    );

    return ListView(
      padding: const EdgeInsets.all(10),
      children: [
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Latters (a~z, A~Z)")),
                validator: (value) => Validator.isAlpha(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Numbers")),
                validator: (value) => Validator.isNumeric(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Letters, Numbers")),
                validator: (value) => Validator.isAlphanumeric(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Before")),
                validator: (value) => Validator.isBefore(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("After")),
                validator: (value) => Validator.isAfter(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("ASCII")),
                validator: (value) => Validator.isAscii(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Multibyte")),
                validator: (value) => Validator.isMultibyte(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Length > 8")),
                validator: (value) => Validator.isLength(value, 8) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Divisible by 3")),
                validator: (value) => Validator.isDivisibleBy(value, 3) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Hexadecimal")),
                validator: (value) => Validator.isHexadecimal(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Hex color")),
                validator: (value) => Validator.isHexColor(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Base64")),
                validator: (value) => Validator.isBase64(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("IP")),
                validator: (value) => Validator.isIP(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Port numeber")),
                validator: (value) => Validator.isPortNumber(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Lowercase")),
                validator: (value) => Validator.isLowercase(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Uppercase")),
                validator: (value) => Validator.isUppercase(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("URL")),
                validator: (value) => Validator.isURL(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Email")),
                validator: (value) => Validator.isEmail(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("FQDN")),
                validator: (value) => Validator.isFQDN(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Credit card")),
                validator: (value) => Validator.isCreditCard(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("ISBN")),
                validator: (value) => Validator.isISBN(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Postal code")),
                validator: (value) => Validator.isPostalCode(value, "CN") ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Surrogate pair")),
                validator: (value) => Validator.isSurrogatePair(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("UUID")),
                validator: (value) => Validator.isUUID(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Contain full width")),
                validator: (value) => Validator.containFullWidth(value) ? null : "Invalid input",
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                autovalidateMode: AutovalidateMode.always,
                decoration: _decoration.copyWith(label: const Text("Contain half width")),
                validator: (value) => Validator.containHalfWidth(value) ? null : "Invalid input",
              ),
            ),
          ],
        ),
      ],
    );
  }
}
