import 'package:flutter/material.dart';
import 'package:text_demo/demo_formatter.dart';
import 'package:text_demo/demo_readable.dart';

import 'demo_validator.dart';

void main() {
  runApp(const DemoApp());
}

class DemoApp extends StatelessWidget {
  const DemoApp({Key? key}) : super(key: key);

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        // This is the theme of your application.
        //
        // Try running your application with "flutter run". You'll see the
        // application has a blue toolbar. Then, without quitting the app, try
        // changing the primarySwatch below to Colors.green and then invoke
        // "hot reload" (press "r" in the console where you ran "flutter run",
        // or simply save your changes to "hot reload" in a Flutter IDE).
        // Notice that the counter didn't reset back to zero; the application
        // is not restarted.
        primarySwatch: Colors.teal,
        brightness: Brightness.dark,
      ),
      home: const DemoPage(),
    );
  }
}

class DemoPage extends StatefulWidget {
  const DemoPage({Key? key}) : super(key: key);

  @override
  State<DemoPage> createState() => _DemoState();
}

class _DemoState extends State<DemoPage> {
  late Map<String, Widget> _demos;
  String? _demoTitle;
  Widget? _demoContent;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_demoTitle ?? "Package Demo"),
      ),
      body: _demoContent ?? const Center(child: Text("Package Demo")),
      drawer: Drawer(
        child: ListView(
          // Important: Remove any padding from the ListView.
          padding: EdgeInsets.zero,
          children: [
            DrawerHeader(
              decoration: const BoxDecoration(
                color: Colors.teal,
              ),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.center,
                children: const [
                  Text('Xinlake Text'),
                  Text("v0.1.0"),
                ],
              ),
            ),
            ..._demos.entries.map((item) {
              return TextButton(
                child: Padding(
                  padding: const EdgeInsets.all(10),
                  child: Text(item.key),
                ),
                onPressed: () => setState(() {
                  _demoTitle = item.key;
                  _demoContent = item.value;
                }),
              );
            }).toList()
          ],
        ),
      ),
    );
  }

  @override
  void initState() {
    super.initState();
    _demos = {
      "Validator": const ValidatorDemo(),
      "Formatter": const FormatterDemo(),
      "Readable": const ReadableDemo(),
    };
    _demoTitle = "Validator";
    _demoContent = _demos[_demoTitle];
  }
}
